import * as THREE from 'three';
import { EventDispatcher } from 'three';
import { GLTFLoader } from 'three/examples/jsm/loaders/GLTFLoader';
import { DRACOLoader } from 'three/examples/jsm/loaders/DRACOLoader';
import { TransformControls } from 'three/examples/jsm/controls/TransformControls';

import texture1 from '../assets/textures/texture1.jpg';
import texture2 from '../assets/textures/texture2.jpg';
import texture3 from '../assets/textures/texture3.jpg';
import texture4 from '../assets/textures/texture4.jpg';

import sampleRoom1 from '../assets/sample_rooms/image1.jpg';
import sampleRoom2 from '../assets/sample_rooms/image2.jpg';
import sampleRoom3 from '../assets/sample_rooms/image3.jpg';
import sampleRoom4 from '../assets/sample_rooms/image4.jpg';


import blind_animation from '../assets/model/blind_animation.glb';
import blind_new from '../assets/model/blind_new.glb';

const imageSize = { x: 400, y: 300 };
const imageHalfWidth = imageSize.x / 2;
const imageHalfHeight = imageSize.y / 2;
const imageResolution = 75;
export class SceneHandler extends EventDispatcher {
    static raycaster = new THREE.Raycaster();
    static scene = undefined;
    static camera = undefined;
    static renderer = undefined;
    static finalVectorArray = [];
    static mixerList = [];
    static animationsList = [];
    static modelsList = new THREE.Group();
    static gltfLoader = new GLTFLoader().setDRACOLoader(new DRACOLoader().setDecoderPath('../js/draco/gltf/'));
    static textureLoader = new THREE.TextureLoader();
    static clock = new THREE.Clock();
    static transforControl = undefined;
    

    static texturesList = [
        texture1,
        texture2,
        texture3,
        texture4,
    ];

    static roomsList = [
        sampleRoom1,
        sampleRoom2,
        sampleRoom3,
        sampleRoom4,
    ]

    static blindList = [
        blind_animation,
        blind_new,
    ]
    static currentRoomImage = this.roomsList[1];
    static currentBlindModel = undefined;

    static index = 0;
    static sampleRoomIndex = 0;
    static blindIndex = 0;

    static setupScene(three) {
        this.scene = three.scene;
        this.camera = three.camera;
        this.renderer = three.gl;
        this.renderer.shadowMap.enabled = true; // Enable shadows
        this.renderer.shadowMap.type = THREE.PCFSoftShadowMap;
        this.raycaster = three.raycaster;
        this.scene.add(this.modelsList);
        // this.transforControl = new TransformControls(this.camera,this.renderer.domElement);
        // this.scene.add(this.transforControl);
        // this.transforControl.addEventListener("change",()=>
        // {
        //     console.log(this.transforControl.object.position.clone(),this.transforControl.object.rotation.clone());
        // });
        window.addEventListener('keydown', function (event) {
            switch (event.code) {
                case 'KeyG':
                    SceneHandler.transforControl.mode = 'translate';
                    break
                case 'KeyR':
                    SceneHandler.transforControl.mode = 'rotate';
                    break
                case 'KeyS':
                    SceneHandler.transforControl.mode = 'scale';
                    break
            }
        })
        // this.createBlinds();
        this.addShadowPlane();
        this.addLight();
        this.animate();

        // this.renderer.domElement.addEventListener("mousedown",()=>
        // {
        //     console.log("clicked");
        //     SceneHandler.playAnimation(true);
        // })

    }

    static setBaseModel()
    {

    }

    static addLight()
    {
        const directionalLight = new THREE.DirectionalLight(0xffffff, 0.2);
        directionalLight.position.set(0, 3, -2);
        directionalLight.castShadow = true;
        directionalLight.shadow.camera.near = 0.1; 
        directionalLight.shadow.camera.far = 30;
        directionalLight.target.position.set(0, 0, 0);
        SceneHandler.scene.add(directionalLight);
    }

    static addShadowPlane()
    {
        const planeGeometry = new THREE.PlaneGeometry(5,2);
        const loader = new THREE.TextureLoader();
      
        const planeMaterial = new THREE.ShadowMaterial({ opacity: 0.2 });
        // const planeMaterial =  new THREE.MeshPhongMaterial({ color:0x00ff00 });
       
        planeMaterial.side = THREE.DoubleSide;
        const plane = new THREE.Mesh(planeGeometry, planeMaterial);
        plane.rotateX(-1.07);
        plane.position.set( 0,-1.2913098317295773,0.6473203737076866) ;
        plane.scale.y = 0.5;
        plane.scale.x = 0.8;
        plane.receiveShadow = true;
        SceneHandler.scene.add(plane);
    }


    static fetchData = async (image) => {
        console.log(image);
        try {
            const blob = await fetch(image).then((response) => response.blob());
            const reader = new FileReader();
            reader.onload = function (event) {
                const base64Image = event.target.result.split(',')[1];

                const apiUrl = 'https://mukesh3444-ai-detect.hf.space/?logs=container';
                const payload = {
                    img: base64Image,
                };
                const jsonPayload = JSON.stringify(payload);
                const headers = {
                    'Content-Type': 'application/json',
                };
                const requestOptions = {
                    method: 'POST',
                    headers: headers,
                    body: jsonPayload,
                };

                fetch(apiUrl, requestOptions, {
                    mode: 'no-cors',
                })
                    .then((response) => {
                        if (!response.ok) {
                            throw new Error(`HTTP Error! Status: ${response.status}`);
                        }
                        return response.json();
                    })
                    .then((data) => {
                        SceneHandler.createBlindsData(data.windows_vertices)
                       
                    })
                    .catch((error) => {
                        console.error('Error:', error);
                    });
            };
            reader.readAsDataURL(blob);
        } catch (error) {
            console.error('Error fetching or processing API data:', error);
        }
    }

    static sortXYPointsForRectangle = (points) => {
        let centerX = 0;
        let centerY = 0;
        for (const [x, y] of points) {
            centerX += parseInt(x);
            centerY += parseInt(y);
        }
        centerX /= points.length;
        centerY /= points.length;

        points.sort(([x1, y1], [x2, y2]) => {
            const angle1 = Math.atan2(y1 - centerY, x1 - centerX);
            const angle2 = Math.atan2(y2 - centerY, x2 - centerX);
            return angle1 - angle2;
        });
    }

    static createBlindsData = (apiData) => {
        let scale = 0.02;
        // if (modelName === "blind_animation") scale = 1;
        let arrayPixelPoints = JSON.parse(JSON.stringify(apiData));

        // Create a temporary array to collect values
        const tempArray = [];

        for (let i = 0; i < arrayPixelPoints.length; i++) {
            let arrayvector = [];
            SceneHandler.sortXYPointsForRectangle(arrayPixelPoints[i]);
            arrayPixelPoints[i].push(arrayPixelPoints[i][0]);

            arrayPixelPoints[i].forEach((item) => {
                let vector = new THREE.Vector3(
                    (parseInt(item[0]) - imageHalfWidth) / imageResolution,
                    -(parseInt(item[1]) - imageHalfHeight) / imageResolution,
                    0.05
                );
                arrayvector.push(vector);
            });

            tempArray.push([arrayvector[0], arrayvector[2], scale]);
            // console.log(arrayvector[0], arrayvector[2], scale);
        }

        SceneHandler.finalVectorArray = tempArray;
        SceneHandler.createBlinds();
       
    };
    

    
    static playAnimation(status)
    {
        console.log("animation triggering");
        this.animationsList.forEach(item=>
        {
            if(status)
            {
                if(!item.paused && item.name.includes("Open"))
                {
                    item.timeScale = 1;
                    item.play();
                    setTimeout(() => {
                        item.paused = true;
                    }, 1800);
                }

                else if(item.paused && item.name.includes("Open"))
                {
                    item.timeScale = 1;
                    item.paused = false;
                    setTimeout(() => {
                        item.paused = true;
                    }, 500);
                }

                
            }

            else if(!status)
            {
                item.timeScale = -1;
                if(!item.paused && item.name.includes("Open"))
                {
                    item.play();
                    setTimeout(() => {
                        
                        item.paused = true;
                    }, 1800);
                }

                else if(item.paused && item.name.includes("Open"))
                {
                    item.paused = false;
                    setTimeout(() => {
                        item.paused = true;
                    }, 500);
                }

                
            }
        })
    }

    static changeTexture()
    {

        const newTexture = SceneHandler.textureLoader.load(this.texturesList[SceneHandler.index]);
        // backgroundTexture.repeat = new THREE.Vector2(2,2)
        SceneHandler.modelsList.traverse(item=>
            {
                if(item.isMesh)
                {
                    item.material.color = null;
                    item.material.map = newTexture;
                    item.material.needsUpdate = true;
                }
            });
        SceneHandler.index++;
    }

    static animate() {
        requestAnimationFrame(SceneHandler.animate);
        if( SceneHandler.mixerList.length > 0)
        {
            let delta = SceneHandler.clock.getDelta();
            SceneHandler.mixerList.forEach((mixer) => {
                mixer.update(delta);
            });
        }
       
    }

    static changeSampleRoom()
    {
        if(this.sampleRoomIndex < this.roomsList.length)
        {
           
            this.currentRoomImage = this.roomsList[SceneHandler.sampleRoomIndex];
            SceneHandler.fetchData(SceneHandler.currentRoomImage);
            const backgroundTexture = SceneHandler.textureLoader.load(this.roomsList[SceneHandler.sampleRoomIndex]);
            let bgmesh = SceneHandler.scene.getObjectByName("bg");
            if(!bgmesh) return;
            bgmesh.material.map = backgroundTexture;
            bgmesh.material.needsUpdate = true;
            this.sampleRoomIndex ++;
        }

        else{
            this.sampleRoomIndex = 0;
        }
    }

    static updateBlinds()
    {
        if(SceneHandler.modelsList.children.length > 0 )
        {

            SceneHandler.modelsList.children.map((item, index)=>
            {
                let pointObj = this.finalVectorArray[index];
                let point = pointObj[0];
                let lastPt = pointObj[1];
                let scale = pointObj[2];

                // console.log(pointObj,item);

                const boundingBox = new THREE.Box3().setFromObject(item);
                const size = new THREE.Vector3();
                boundingBox.getSize(size);
                let actualWidth = lastPt.x - point.x;
                let actualHeight = lastPt.y - point.y;
        
                item.scale.set(actualWidth / size.x, Math.abs(actualHeight / size.y), scale);
                item.position.set((point.x + ((lastPt.x - point.x) / 2))+0.05, lastPt.y, point.z+0.08);
                if(index == 0) item.position.x = item.position.x - 0.1;
            });
        }
    }
    
    static emptyGroup() {
        while (SceneHandler.modelsList.children.length > 0) {
          let child = SceneHandler.modelsList.children[0];
          SceneHandler.modelsList.remove(child);
        }
      }

    static deleteAllBlindDetails()
    {
        SceneHandler.emptyGroup();
        SceneHandler.animationsList = [];
        SceneHandler.mixerList = [];
    }

    static changeBlind()
    {
        if(this.blindIndex < this.blindList.length)
        {
            console.log(this.blindList[this.blindIndex]);
            SceneHandler.currentBlindModel = this.blindList[this.blindIndex];
            SceneHandler.createBlinds();
            this.blindIndex ++;
        }

        else{
            this.blindIndex = 0;
        }
    }

    static createBlinds()
    {
        SceneHandler.deleteAllBlindDetails();
        if(this.finalVectorArray.length > 0 )
        {

            this.finalVectorArray.map((item, index)=>
            {
                let windowScene = undefined;
                let mixer = undefined;
                let animations = undefined;
                this.gltfLoader.load(SceneHandler.currentBlindModel, function (gltf)
                {
                    let point = item[0];
                    let lastPt = item[1];
                    let scale = item[2];
                    console.log(item);

                    windowScene = gltf.scene;

                    windowScene.traverse(item=>
                        {
                            if(item.isMesh)
                            {
                                item.castShadow = true;
                                item.receiveShadow = true;
                            }
                        })
                    windowScene.castShadow = true;
                    windowScene.receiveShadow = true;
                    animations = gltf.animations;
                    console.log(animations);

                   
                    mixer = new THREE.AnimationMixer(gltf.scene);
                    SceneHandler.mixerList.push(mixer);

                    const boundingBox = new THREE.Box3().setFromObject(windowScene);
                    const size = new THREE.Vector3();
                    boundingBox.getSize(size);
                    let actualWidth = lastPt.x - point.x;
                    let actualHeight = lastPt.y - point.y;
                    windowScene.scale.set(actualWidth / size.x, Math.abs(actualHeight / size.y), scale);
                    windowScene.position.set((point.x + ((lastPt.x - point.x) / 2))+0.05, lastPt.y, point.z+0.08);
                    if(index == 0) windowScene.position.x = windowScene.position.x - 0.1;
                    if(animations.length > 0)
                    {
                        animations.forEach(clip=>
                        {
                            let action = mixer.clipAction(clip);
                            action.name = clip.name;
                            action.loop = THREE.LoopOnce;
                            action.clampWhenFinished = true;
                            SceneHandler.animationsList.push(action);
                        })
                       
                    }
                    SceneHandler.modelsList.add(windowScene);
                });
            })
        }
    }

}