import React, { useState, useEffect, useCallback } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { Image, FormControl, InputGroup } from 'react-bootstrap';
import { Search } from 'react-bootstrap-icons';
import { updateSavedProject } from '@store/actions/saved-project-actions';
import { fetchColorsAPIData } from '@store/actions/colors-api-actions';
import { getEndpointBaseURL } from '@services/environment';

import './colors-texture.scss';


const ColorsTexture = () => {
  const [searchText, setSearchText] = useState('');
  const dispatch = useDispatch();
  const colorsDispatch = useDispatch();
  const defaultProduct = useSelector((state) => state.savedProjects.defaultProduct);
  const savedSceneDetails = useSelector((state) => state.savedScene);
  const colorsAPIData = useSelector((state) => state.colorAPIData);
  const colorOptionsGroup = useSelector((state) => state.colorsSelection.colorOptionsGroup);
  const [colorsData, setColorsData] = useState()

  useEffect(() => {
    const url = `${getEndpointBaseURL()}color/${defaultProduct?.id}`;
    colorsDispatch(fetchColorsAPIData(url));
  }, []);

  useEffect(() => {
    const filteredData = colorOptionsGroup?.map(item => {
      return item.id
    });

    const filteredItems = colorsAPIData?.colorsData?.filter(item => {
      const colorFilterOptions = item.color_filter_options_id.split(',').map(Number);
      return filteredData.some(option => {
    if(colorFilterOptions.includes(option)){
      return item
    }
      });
    });

    setColorsData(filteredItems);
  }, [defaultProduct]);


  const handleSearch = (e) => setSearchText(e.target.value);

  const search = (items) => {
    return items?.filter((item) => {
        return ['color_name'].some((newItem) => {
            return (
                item[newItem]
                  .toString()
                  .toLowerCase()
                  .indexOf(searchText.toLowerCase()) > -1
            );
        });
    });
  }

  const handleClick = useCallback((item) => {
    try {
      const options = { ...defaultProduct, color: { ...item } };
      if (savedSceneDetails.currentScene && 'color' in options && 'file_path' in item) {
        savedSceneDetails.currentScene.changeTexture(item.file_path);
      }
      dispatch(updateSavedProject(options));
    } catch (error) {
      console.error('Error handling click:', error);
    }
  }, [defaultProduct, savedSceneDetails.currentScene]);

  const clearSearchResult = () => setSearchText('');

  return (
    <>
      {colorsData && (
        <>
          <InputGroup className='search-box'>
            <FormControl value={searchText} onChange={handleSearch} />
            <Search className='search-icon' size={18} />
            <button className='search-clear' onClick={clearSearchResult}>X</button>
          </InputGroup>

          <div className="color-length">
          {
            search(colorsData)?.length > 0
            ? search(colorsData)?.length + ' '
            : "No "
          }
            color(s) match the filters.
        </div>

        <div className='color-container'>
          <ul className='color-aside' style={{ listStyle: 'none', cursor: 'pointer' }}>
            {search(colorsData).map((item, idx) => (
              <li key={idx}>
                <Image src={item.file_path} onClick={() => handleClick(item)} />
                <p>{item.color_name}</p>
              </li>
            ))}
          </ul>
        </div>
        </>
      )}
    </>
  );
};

export default ColorsTexture;
